;	======================================
;	       Farbwechsler mit Tiny12
;	======================================

;	-------------------------------
;	Port
;	-------------------------------
;	B0: Output LED red
;	B1: Output LED green
;	B2: Output LED blue
;	B3: Input active high stop color change
;	B4: Input active high all on

;	-------------------------------
;	Timer
;	-------------------------------
;	Timer0: Interrupt every 0,213 ms Prescaler 1

;	---------------------------------
;	Standard definitions for ATtiny12
;	---------------------------------
	.include "tn12def.inc"

;	-------------------------------
;	Constant definitions
;	-------------------------------
	.equ	pwmmax	=63		;max. PWM pulses
	.equ	wc1max	=255		;end value wait counter 1
	.equ	wc2max	=2		;end value wait counter 2
	.equ	port	=portb
	.equ	pin	=pinb
	.equ	ddr	=ddrb
	.equ	out1	=0		;Red
	.equ	out2	=1		;Green
	.equ	out3	=2		;Blue
	.equ	in1	=3		;S1 Stop changing
	.equ	in2	=4		;S2 All on

;	-------------------------------
;	Register definitions
;	-------------------------------
	.def	stor	=r0		;storage variable for LPM
	.def	flags	=r1		;flag storage
	.def	temp	=r16		;temporary storage variable 1
	.def	temp2	=r17		;temporary storage variable 2
	.def	led1	=r18		;PWM LED red
	.def	led2	=r19		;PWM LED green
	.def	led3	=r20		;PWM LED blue
	.def	pwm	=r22		;number PWM pulses
	.def	waitco	=r23		;wait counter variable
	.def	waitco2	=r24		;wait counter variable 2
	.def	loop	=r25

;	---------------------------------
;	Program start, Interrupt vectors
;	---------------------------------
	.cseg
	.org	0x0000

reset:	rjmp	initia			;program start
exint0:	reti
pncnge:	reti
t0ovfl:	rjmp	timer0			;time counter
eerdy:	reti
ancomp:	reti

;Initialize I/O and interrupts
initia:	cli				;disable interrupts
	ldi	temp,0x49
	out	osccal,temp		;calibrate oscillator
	ldi	temp,0x07
	out	ddr,temp		;3 outputs, 2 inputs
	ldi	temp,0x18
	out	port,temp		;activate pull-ups
	ldi	temp,1<<toie0
	out	timsk,temp		;Timer0 interrupt
	ldi	temp,1<<cs00
	out	tccr0,temp		;Timer0 overflow
	sei				;enable interrupts

	clr	temp
	clr	temp2
	clr	loop

;Main program
main:	sbic	pin,in1
	rjmp	main

	cpi	loop,0		;red -> yellow
	brne	main2
	ldi	led1,pwmmax
	mov	led2,temp2
	ldi	led3,0
	cpi	temp2,pwmmax
	breq	main2a
	inc	temp2
	rjmp	mend

main2a:	inc	loop
	clr	temp2
main2:	cpi	loop,1		;yellow -> white
	brne	main3
	ldi	led1,pwmmax
	ldi	led2,pwmmax
	mov	led3,temp2
	cpi	temp2,pwmmax
	breq	main3a
	inc	temp2
	rjmp	mend

main3a:	inc	loop
	ldi	temp2,pwmmax
main3:	cpi	loop,2		;white -> cyan
	brne	main4
	mov	led1,temp2
	ldi	led2,pwmmax
	ldi	led3,pwmmax
	cpi	temp2,0
	breq	main4a
	dec	temp2
	rjmp	mend

main4a:	inc	loop
	ldi	temp2,pwmmax
main4:	cpi	loop,3		;cyan -> blue
	brne	main5
	ldi	led1,0
	mov	led2,temp2
	ldi	led3,pwmmax
	cpi	temp2,0
	breq	main5a
	dec	temp2
	rjmp	mend

main5a:	inc	loop
	clr	temp2
main5:	cpi	loop,4		;blue -> magenta
	brne	main6
	mov	led1,temp2
	ldi	led2,0
	ldi	led3,pwmmax
	cpi	temp2,pwmmax
	breq	main6a
	inc	temp2
	rjmp	mend

main6a:	inc	loop
	clr	temp2
main6:	cpi	loop,5		;magenta -> white
	brne	main7
	ldi	led1,pwmmax
	mov	led2,temp2
	ldi	led3,pwmmax
	cpi	temp2,pwmmax
	breq	main7a
	inc	temp2
	rjmp	mend

main7a:	inc	loop
	ldi	temp2,pwmmax
main7:	cpi	loop,6		;white -> yellow
	brne	main8
	ldi	led1,pwmmax
	ldi	led2,pwmmax
	mov	led3,temp2
	cpi	temp2,0
	breq	main8a
	dec	temp2
	rjmp	mend

main8a:	inc	loop
	ldi	temp2,pwmmax
main8:	cpi	loop,7		;yellow -> green
	brne	main9
	mov	led1,temp2
	ldi	led2,pwmmax
	ldi	led3,0
	cpi	temp2,0
	breq	main9a
	dec	temp2
	rjmp	mend

main9a:	inc	loop
	clr	temp2
main9:	cpi	loop,8		;green -> cyan
	brne	maina
	ldi	led1,0
	ldi	led2,pwmmax
	mov	led3,temp2
	cpi	temp2,pwmmax
	breq	mainaa
	inc	temp2
	rjmp	mend

mainaa:	inc	loop
	ldi	temp2,0
maina:	cpi	loop,9		;cyan -> white
	brne	mainb
	mov	led1,temp2
	ldi	led2,pwmmax
	ldi	led3,pwmmax
	cpi	temp2,pwmmax
	breq	mainba
	inc	temp2
	rjmp	mend

mainba:	inc	loop
	ldi	temp2,pwmmax
mainb:	cpi	loop,10		;white -> magenta
	brne	mainc
	ldi	led1,pwmmax
	mov	led2,temp2
	ldi	led3,pwmmax
	cpi	temp2,0
	breq	mainca
	dec	temp2
	rjmp	mend

mainca:	inc	loop
	ldi	temp2,pwmmax
mainc:	cpi	loop,11		;magenta -> red
	brne	mend
	ldi	led1,pwmmax
	ldi	led2,0
	mov	led3,temp2
	cpi	temp2,0
	breq	mve
	dec	temp2
	rjmp	mend

mve:	clr	loop
	clr	temp2

mend:	clr	waitco2
wait1:	clr	waitco
wait2:	cpi	waitco,wc1max
	brlo	wait2
	inc	waitco2
	cpi	waitco2,wc2max
	brlo	wait1
	rjmp	main

timer0: ;Timer0 ISR, use flags, pwm only here
	in	flags,sreg	;save flags
	inc	waitco
rot:	sbic	pin,in2		;All on?
	rjmp	rot4
	cpi	led1,0
	breq	rot3
	cpi	led1,pwmmax
	breq	rot4
	cp	pwm,led1
	brlo	rot4
rot3:	cbi	port,out1
	rjmp	gruen
rot4:	sbi	port,out1
gruen:	sbic	pin,in2		;All on?
	rjmp	gruen4
	cpi	led2,0
	breq	gruen3
	cpi	led2,pwmmax
	breq	gruen4
	cp	pwm,led2
	brlo	gruen4
gruen3:	cbi	port,out2
	rjmp	blau
gruen4:	sbi	port,out2
blau:	sbic	pin,in2		;All on?
	rjmp	blau4
	cpi	led3,0
	breq	blau3
	cpi	led3,pwmmax
	breq	blau4
	cp	pwm,led3
	brlo	blau4
blau3:	cbi	port,out3
	rjmp	ende
blau4:	sbi	port,out3

ende:	inc	pwm			;increment PWM counter
	cpi	pwm,pwmmax
	brne	ende2
	clr	pwm			;clear PWM counter
ende2:	out	sreg,flags		;restore flags
	reti